import logging
from logging import NullHandler

from shopscraper.api import read_json, scrape, scrape_to_json
from shopscraper.prod_objs import Image, Product, Variant

logging.getLogger(__name__).addHandler(NullHandler())


# def add_stderr_logger(level: int = logging.DEBUG) -> logging.StreamHandler:
#     """
#     Helper for quickly adding a StreamHandler to the logger. Useful for
#     debugging.
#     Returns the handler after adding it.
#     """
#     # This method needs to be in this __init__.py to get the __name__ correct
#     # even if urllib3 is vendored within another package.
#     logger = logging.getLogger(__name__)
#     handler = logging.StreamHandler()
#     handler.setFormatter(logging.Formatter("%(asctime)s %(levelname)s %(message)s"))
#     logger.addHandler(handler)
#     logger.setLevel(level)
#     logger.debug("Added a stderr logging handler to logger: %s", __name__)
#     return handler


# ... Clean up.
del NullHandler
